package ch.ethz.matsim.projects.astra_2018_002.flow_efficiency;

import org.matsim.api.core.v01.Id;
import org.matsim.api.core.v01.Scenario;
import org.matsim.api.core.v01.population.Person;
import org.matsim.core.config.Config;
import org.matsim.core.config.groups.QSimConfigGroup.VehiclesSource;
import org.matsim.vehicles.Vehicle;
import org.matsim.vehicles.VehicleType;
import org.matsim.vehicles.VehicleUtils;

import ch.ethz.matsim.projects.astra_2018_002.private_av.PrivateAVConfigGroup;
import ch.ethz.matsim.projects.astra_2018_002.shared_av.SharedAVConfigGroup;

public class FlowEfficiencyConfigurator {
	static final public Id<VehicleType> CONVENTIONAL_VEHICLE_TYPE_ID = Id.create("car", VehicleType.class);
	static final public Id<VehicleType> PRIVATE_AV_VEHICLE_TYPE_ID = Id.create("private_av", VehicleType.class);
	static final public Id<VehicleType> SHARED_AV_VEHICLE_TYPE_ID = Id.create("shared_av", VehicleType.class);
	static final public Id<VehicleType> PRAV3_ID = Id.create("prav3", VehicleType.class);
	static final public Id<VehicleType> PRAV4_ID = Id.create("prav4", VehicleType.class);
	static final public Id<VehicleType> PRAV5_ID = Id.create("prav5", VehicleType.class);
	static final public Id<VehicleType> PASSENGER_VEHICLE_TYPE_ID = Id.create("car_passenger", VehicleType.class);

	static public void configure(Config config) {
		// Load vehicles by vehicle type
		config.qsim().setVehiclesSource(VehiclesSource.modeVehicleTypesFromVehiclesData);

		PrivateAVConfigGroup privateConfig = (PrivateAVConfigGroup) config.getModules()
				.get(PrivateAVConfigGroup.GROUP_NAME);
		SharedAVConfigGroup sharedConfig = (SharedAVConfigGroup) config.getModules()
				.get(SharedAVConfigGroup.GROUP_NAME);

		if (privateConfig.getUsePrivateAVs() || sharedConfig.getUseSharedAVs()) {
			// With the AvFlowEfficiencyCalculator we can distinguish between HLS
			// and other roads in combination with vehicle types
			FlowEfficiencyCalculator calculator = new AvFlowEfficiencyCalculator();
			FlowEfficiencyCalculator.INSTANCE.set(calculator);
		} else {
			// Set the calculator instance (not the best design, but works)
			FlowEfficiencyCalculator calculator = new StaticFlowEfficiencyCalculator();
			FlowEfficiencyCalculator.INSTANCE.set(calculator);
		}
	}

	static public void defineVehicleTypes(Scenario scenario) {
		VehicleType carVehicleType = VehicleUtils.getFactory() //
				.createVehicleType(CONVENTIONAL_VEHICLE_TYPE_ID);
		scenario.getVehicles().addVehicleType(carVehicleType);

		VehicleType passengerVehicleType = VehicleUtils.getFactory() //
				.createVehicleType(PASSENGER_VEHICLE_TYPE_ID);
		scenario.getVehicles().addVehicleType(passengerVehicleType);

		VehicleType privateAvVehicleType = VehicleUtils.getFactory() //
				.createVehicleType(PRIVATE_AV_VEHICLE_TYPE_ID);
		scenario.getVehicles().addVehicleType(privateAvVehicleType);

		VehicleType sharedAvVehicleType = VehicleUtils.getFactory() //
				.createVehicleType(SHARED_AV_VEHICLE_TYPE_ID);
		scenario.getVehicles().addVehicleType(sharedAvVehicleType);

		VehicleType prav3VehicleType = VehicleUtils.getFactory() //
				.createVehicleType(PRAV3_ID);
		scenario.getVehicles().addVehicleType(prav3VehicleType);

		VehicleType prav4VehicleType = VehicleUtils.getFactory() //
				.createVehicleType(PRAV4_ID);
		scenario.getVehicles().addVehicleType(prav4VehicleType);

		VehicleType prav5VehicleType = VehicleUtils.getFactory() //
				.createVehicleType(PRAV5_ID);
		scenario.getVehicles().addVehicleType(prav5VehicleType);

		for (Person person : scenario.getPopulation().getPersons().values()) {
			Id<Vehicle> vehicleId = Id.createVehicleId(person.getId().toString() + "_car_passenger");
			Vehicle vehicle = scenario.getVehicles().getFactory().createVehicle(vehicleId, passengerVehicleType);
			scenario.getVehicles().addVehicle(vehicle);
		}
	}
}
