package ch.ethz.matsim.projects.astra_2018_002.mode_choice.cost;

import ch.ethz.matsim.projects.astra_2018_002.mode_choice.base.PersonVariables;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.base.TripVariablesPt;
import ch.ethz.matsim.projects.astra_2018_002.shared_av.pricing.SharedAVTravelCost;

public class CostModel {
	private final CostParameters parameters;
	private final SharedAVTravelCost shared;

	public CostModel(CostParameters parameters, SharedAVTravelCost shared) {
		this.parameters = parameters;
		this.shared = shared;
	}

	public double calculateCarTravelCost(double distance_km) {
		return parameters.carCostPerKm_CHF * distance_km;
	}

	public double calculatePrivateAvTravelCost(double distance_km) {
		return parameters.privateAvCostPerKm_CHF * distance_km;
	}

	public double calculateSharedAvTravelCost(double distance_km, boolean isPooled) {
		return shared.getTravelCost(distance_km, isPooled);
	}

	public double calculatePtTravelCost(PersonVariables personVariables, TripVariablesPt tripVariables) {
		/*
		 * TODO: Later on, we should have a more complex cost model here.
		 */

		if (personVariables.hasGeneralSubscription) {
			return 0.0;
		}

		if (personVariables.hasRegionalSubscription) {
			if (tripVariables.originHomeDistance_km <= parameters.ptRegionalRadius_km
					&& tripVariables.destinationHomeDistance_km <= parameters.ptRegionalRadius_km) {
				return 0.0;
			}
		}

		double fullCost = Math.max(parameters.ptCostMinimum_CHF,
				parameters.ptCostPerKm_CHF * tripVariables.inVehicleDistance_km);

		if (personVariables.hasHalbtaxSubscription) {
			return fullCost * 0.5;
		} else {
			return fullCost;
		}
	}
}
