package ch.ethz.matsim.projects.astra_2018_002.mode_choice.utils;

import org.matsim.api.core.v01.Coord;
import org.matsim.api.core.v01.population.Activity;
import org.matsim.core.population.PopulationUtils;

import ch.ethz.matsim.baseline_scenario.config.CommandLine;
import ch.ethz.matsim.baseline_scenario.config.CommandLine.ConfigurationException;
import ch.ethz.matsim.discrete_mode_choice.model.DiscreteModeChoiceTrip;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.SwissDiscreteModeChoiceModule;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.SwissUtilityEstimator;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.SwissUtilityParameters;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.SwissUtilityParameters.UtilitySet;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.base.PersonVariables;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.base.TripVariablesBike;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.base.TripVariablesCar;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.base.TripVariablesPt;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.base.TripVariablesWalk;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.cost.CostModel;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.cost.CostParameters;
import ch.ethz.matsim.projects.astra_2018_002.shared_av.pricing.DefaultSharedAVTravelCost;

public class TestUtility {
	static public void main(String[] args) throws ConfigurationException {
		CommandLine cmd = new CommandLine.Builder(args) //
				.allowPrefixes(SwissDiscreteModeChoiceModule.COMMAND_LINE_PREFIX) //
				.build();

		CostParameters costParameters = new CostParameters();
		CostModel costModel = new CostModel(costParameters, new DefaultSharedAVTravelCost(costParameters));

		SwissUtilityParameters parameters = SwissUtilityParameters.build(UtilitySet.APPENDIX_A5, true, false);
		SwissUtilityParameters.adaptFromCommandLine(parameters, cmd);

		SwissUtilityEstimator estimator = new SwissUtilityEstimator(null, null, null, costModel, null, parameters);

		Activity originActivity = PopulationUtils.getFactory().createActivityFromCoord("home", new Coord(0.0, 0.0));
		Activity destinationActivity = PopulationUtils.getFactory().createActivityFromCoord("home",
				new Coord(0.0, 0.0));
		String initialMode = "unknown";
		double departureTime = 5.0 * 3600.0;

		DiscreteModeChoiceTrip trip = new DiscreteModeChoiceTrip(originActivity, destinationActivity, initialMode,
				departureTime);

		PersonVariables personVariables = new PersonVariables();

		TripVariablesWalk tripVariablesWalk = new TripVariablesWalk();
		double walkUtility = estimator.estimateWalkTrip(trip, personVariables, tripVariablesWalk);

		TripVariablesBike tripVariablesBike = new TripVariablesBike();
		double bikeUtility = estimator.estimateBikeTrip(trip, personVariables, tripVariablesBike);

		TripVariablesPt tripVariablesPt = new TripVariablesPt();
		double ptUtility = estimator.estimatePtTrip(trip, personVariables, tripVariablesPt);

		TripVariablesCar tripVariablesCar = new TripVariablesCar();
		double carUtility = estimator.estimateCarTrip(trip, personVariables, tripVariablesCar);

		System.out.println("Walk utility: " + walkUtility);
		System.out.println("Bike utility: " + bikeUtility);
		System.out.println("Pt utility: " + ptUtility);
		System.out.println("Car utility: " + carUtility);

		double denominator = Math.exp(walkUtility) + Math.exp(bikeUtility) + Math.exp(ptUtility) + Math.exp(carUtility);
		double probability = 0.0;

		probability = Math.exp(walkUtility) / denominator;
		System.out.println("Walk probability: " + probability);

		probability = Math.exp(bikeUtility) / denominator;
		System.out.println("Bike probability: " + probability);

		probability = Math.exp(ptUtility) / denominator;
		System.out.println("Pt probability: " + probability);

		probability = Math.exp(carUtility) / denominator;
		System.out.println("Car probability: " + probability);
	}
}
